"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const configuration_lib_1 = require("./utils/configuration-lib");
const parser_1 = require("./utils/parser");
const webpack_libs_1 = require("./utils/webpack-libs");
/**
 *
 * @param configFilePath
 */
function develop(configFilePath) {
    return __awaiter(this, void 0, void 0, function* () {
        // create a usable configuration
        const configPath = yield configuration_lib_1.prepareConfiguration(configFilePath);
        // parse the configuration for plugins
        const plugins = parser_1.parseForPlugins(configPath, configFilePath);
        console.log("plugins: ", plugins);
        // load the configuration statically (without objects)
        const staticConfig = configuration_lib_1.loadStaticConfiguration(configPath).default;
        console.log("staticConfig: ", staticConfig);
        // parse the loaded configuration in compile mode (statically)
        const parsedConfig = yield parser_1.extractConfigs(staticConfig, plugins, parser_1.INFRASTRUCTURE_MODES.COMPILATION);
        console.log("\n--------------------------------------------------");
        console.log("parsed config: ", parsedConfig);
        console.log("--------------------------------------------------\n");
        // now run the webpacks
        yield Promise.all(parsedConfig.webpackConfigs.map((wpConfig) => __awaiter(this, void 0, void 0, function* () {
            //console.log("wpConfig: ", wpConfig);
            yield webpack_libs_1.runWebpack(wpConfig);
        })));
    });
}
exports.develop = develop;
;
//# sourceMappingURL=develop.js.map