"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const ssr_config_1 = require("./ssr-config");
const libs_1 = require("../libs");
const deepmerge = __importStar(require("deepmerge"));
exports.ISOCONFIG_SERVERNAME = 'server';
function isoToSsr(configFilePath, iso, ssrConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        const fs = require('fs');
        const path = require('path');
        const cmd = require('node-cmd');
        const ReplacePlugin = require('webpack-plugin-replace');
        // get the current path from the main project
        const pwd = yield libs_1.promisify(callback => cmd.get(`pwd`, callback))
            .then((data) => data)
            .catch(err => {
            console.log("err: ", err);
            return "";
        });
        const absolutePath = pwd.toString().replace(/(?:\r\n|\r|\n)/g, "");
        const tempPath = path.join(absolutePath, libs_1.TEMP_FOLDER);
        if (!fs.existsSync(tempPath)) {
            fs.mkdirSync(tempPath);
        }
        ;
        // TODO server hardcoded here!
        const serverPath = path.join(tempPath, "server");
        if (!fs.existsSync(serverPath)) {
            fs.mkdirSync(serverPath);
        }
        const isoConfigPath = path.resolve(absolutePath, configFilePath.replace(/\.[^/.]+$/, ""));
        //console.log("replaceValues: " , replaceValues);
        // the ssrConfig may already contain a serverConfig! merge it
        ssrConfig["serverConfig"] = deepmerge.all([
            ssrConfig.serverConfig !== undefined ? ssrConfig.serverConfig : {},
            {
                /*entry: iso.clientApps.reduce((res, app) => {
                    console.log("processing app: " , app);
                    // add to the entry point
                    if (app.dataLayer !== undefined && app.dataLayer) {
                        console.log("data Layer during config: " ,app.dataLayer );
                        
                        // TODO refactor this, external dependency to DynamoDbGraphQlDataLayer
                        res[app.dataLayer.id] = "./"+path.join("node_modules", "DynamoDbGraphQlDataLayer", "build", "database.js");
                    }
    
                    return res;
                }, {
                    // we start with the main entry point
                    // TODO refactor this!!!!
                    server: "./"+path.join("node_modules", "infrastructure-components", "dist", "iso_src", "server.js"),
                }),*/
                entry: {
                    server: "./" + path.join("node_modules", "infrastructure-components", "dist", "iso_src", "server.js"),
                },
                name: exports.ISOCONFIG_SERVERNAME,
            }
        ]);
        ssrConfig["serverConfig"]["output"] = {
            libraryTarget: "commonjs2",
            path: serverPath,
            filename: '[name].js'
        };
        ssrConfig["serverConfig"]["resolve"] = {
            alias: {
                IsoConfig: isoConfigPath
            }
        };
        ssrConfig["serverConfig"]["target"] = "node";
        console.log("isoToSsr, run webpack on server config");
        // pack the source code of the isomorphic server
        yield libs_1.runWebpack(libs_1.complementWebpackConfig(ssrConfig["serverConfig"]));
        //console.log("resolved: ", resolveAssetsPath(ssrConfig));
        const serverIndexPath = path.join(serverPath, "index.js");
        fs.writeFileSync(serverIndexPath, `const lib = require ('./server');
const server = lib.default('${ssrConfig.assetsPath}', '${ssr_config_1.resolveAssetsPath(ssrConfig)}');
exports.default = server;`);
        //
        console.log('serverindex created...');
        // ATTENTION !! the overwriting the ssrConfig["serverConfig"] here!!!
        /*  NOT REQUIRED ANYMORE for the datalayers have their own entry points now
         entry: iso.clientApps.reduce((res, app) => {
    
    
    
         // add to the entry point
         if (app.dataLayer !== undefined && app.dataLayer) {
         console.log("data layer in server: " , app.dataLayer)
    
         res[app.dataLayer.id] = path.join(serverPath, app.dataLayer.id+".js");
         }
    
         return res;
         }, {
         // we start with the main entry point
         // TODO refactor this!!!!
         index: serverIndexPath,
         }),*/
        ssrConfig["serverConfig"] = {
            name: exports.ISOCONFIG_SERVERNAME,
            entry: serverIndexPath,
        };
        var configs = new Array();
        console.log('now building clients...');
        ssrConfig["clientConfig"] = yield Promise.all(iso.clientApps.map((app, index) => __awaiter(this, void 0, void 0, function* () {
            console.log("index: ", index);
            console.log("app: ", app);
            return yield createClientApp(isoConfigPath, tempPath, iso.clientApps[index], index);
        })));
        console.log('now building data layers...');
        // see: ssr-config --> there these temporary apps are finally built!
        ssrConfig["datalayerConfig"] = yield Promise.all(iso.dataLayers.map((dl, index) => __awaiter(this, void 0, void 0, function* () {
            console.log("index: ", index);
            console.log("dataLayer: ", dl);
            return yield createDataLayer(isoConfigPath, tempPath, iso.dataLayers[index], index);
        })));
        console.log("ISOTOSSR: ssrConfig: ", ssrConfig);
        return ssrConfig;
    });
}
exports.isoToSsr = isoToSsr;
;
/**
 * @param isoConfigPath: the current path of the user's main app, use it to load the default, entry point, then
 * you can load the IsoConfig (runtime) from it and get all the objects you need
 */
function createDataLayer(isoConfigPath, outputPath, dataLayer, index) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('createDataLayer... ', isoConfigPath);
        const fs = require('fs');
        const path = require('path');
        const ReplacePlugin = require('webpack-plugin-replace');
        const dlPath = path.join(outputPath, dataLayer.id);
        if (!fs.existsSync(dlPath)) {
            fs.mkdirSync(dlPath);
        }
        const filename = dataLayer.id + ".js";
        const datalayerIndexPath = path.join(dlPath, filename);
        // pack the source code of the isomorphic client
        yield libs_1.runWebpack(libs_1.complementWebpackConfig({
            entry: {
                // TODO refactor this, external dependency to DynamoDbGraphQlDataLayer
                dataLayer: "./" + path.join("node_modules", "ddb-gql-data-layer", "build", "database.js")
            },
            output: {
                libraryTarget: "commonjs2",
                path: dlPath,
                filename: filename
            },
            resolve: {
                alias: {
                    IsoConfig: isoConfigPath
                }
            },
            target: "node",
            plugins: [
                new ReplacePlugin({
                    values: {
                        '"INDEX_OF_DATALAYER"': index
                    }
                })
            ]
        }));
        //console.log(createServer(iso, ssrConfig).toString());
        //return;
        // create an index-file for the server:
        //console.log("iso: ", iso)
        return {
            entry: datalayerIndexPath,
            name: dataLayer.id
        };
    });
}
function createClientApp(isoConfigPath, outputPath, app, index) {
    return __awaiter(this, void 0, void 0, function* () {
        console.log('createClientApp... ', isoConfigPath);
        const fs = require('fs');
        const path = require('path');
        const clientPath = path.join(outputPath, app.id);
        if (!fs.existsSync(clientPath)) {
            fs.mkdirSync(clientPath);
        }
        const filename = app.id + ".js";
        const clientIndexPath = path.join(clientPath, filename);
        const ReplacePlugin = require('webpack-plugin-replace');
        // pack the source code of the isomorphic client
        yield libs_1.runWebpack(libs_1.complementWebpackConfig({
            entry: {
                // TODO refactor this!!!!
                client: "./" + path.join("node_modules", "infrastructure-components", "dist", "iso_src", "client.js")
            },
            output: {
                path: clientPath,
                filename: filename
            },
            resolve: {
                alias: {
                    IsoConfig: isoConfigPath
                }
            },
            target: "web",
            plugins: [
                new ReplacePlugin({
                    values: {
                        '"INDEX_OF_CLIENT"': index
                    }
                })
            ]
        }));
        //console.log(createServer(iso, ssrConfig).toString());
        //return;
        // create an index-file for the server:
        //console.log("iso: ", iso)
        return {
            entry: clientIndexPath,
            name: app.id
        };
    });
}
/*
export async function startIso (isoConfig: IsoConfig, ssrConfig: SsrConfig, keepSlsYaml: boolean) {

    // prepare the sls-config
    const slsConfig = toSlsConfig(ssrConfig.stackName, ssrConfig.serverConfig, ssrConfig.buildPath);

    // start the sls-config
    startSlsOffline(slsConfig, keepSlsYaml);

}*/
//# sourceMappingURL=iso-config.js.map