"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const yaml_edit_1 = require("../yaml-edit");
const libs_1 = require("../libs");
/**
 * Basic serveless.yml frame
 */
exports.SERVERLESS_YML = `service:

plugins:
  # allows running the stack locally on the dev-machine
  - serverless-offline
  - serverless-pseudo-parameters
  
# the custom section
custom:
  # allows accessing the offline backend when using Docker
  serverless-offline:
    host: 0.0.0.0
    port: \${self:provider.PORT, env:PORT, '3000'}

package:

provider:
  name: aws
  runtime: nodejs8.10
  
functions:

resources:

`;
function createSlsYaml(slsConfig, keepSlsYaml) {
    return __awaiter(this, void 0, void 0, function* () {
        // create a serverless.yml in the root directory
        const fs = require("fs");
        const ymlExists = yield existsSlsYml();
        if (!keepSlsYaml && ymlExists) {
            console.error("serverless.yml already exists. This file would be overwritten! Please remove it before proceeding.");
            return;
        }
        const cmd = require('node-cmd');
        let yamlEdit = yaml_edit_1.YamlEditor(exports.SERVERLESS_YML);
        // add the sls-configuration to the yml
        Object.keys(slsConfig).forEach(key => {
            //console.log (key);
            yamlEdit.insertChild(key, slsConfig[key]);
        });
        let yamlString = yamlEdit.dump();
        //console.log(yamlString);
        yield fs.writeFile('serverless.yml', yamlString, function (err) {
            if (err)
                throw err;
            console.log('serverless.yml created...');
        });
    });
}
exports.createSlsYaml = createSlsYaml;
function existsSlsYml() {
    const fs = require("fs");
    try {
        fs.statSync('serverless.yml');
    }
    catch (error) {
        return false;
    }
    return true;
}
exports.existsSlsYml = existsSlsYml;
function runSlsCmd(slsCmd, onStdOut) {
    return __awaiter(this, void 0, void 0, function* () {
        const cmd = require('node-cmd');
        return new Promise((resolve, reject) => {
            let data_line = '';
            const processRef = cmd.get(slsCmd);
            processRef.stdout.on('data', function (data) {
                return __awaiter(this, void 0, void 0, function* () {
                    data_line += data;
                    if (data_line[data_line.length - 1] == '\n') {
                        console.log(data_line);
                        if (onStdOut !== undefined) {
                            onStdOut(data_line);
                        }
                    }
                });
            });
            processRef.on("exit", function (code, signal) {
                console.log('child process exited with ' + `code ${code} and signal ${signal}`);
                resolve();
            });
        });
    });
}
function deploySls(slsConfig, keepSlsYaml) {
    return __awaiter(this, void 0, void 0, function* () {
        const fs = require("fs");
        createSlsYaml(slsConfig, keepSlsYaml);
        // login in to Serverless/Aws
        yield libs_1.slsLogin();
        //  sls deploy && node -r dotenv/config -e 'require(\"./src/config\").s3sync()' dotenv_config_path=.dev.env",
        yield runSlsCmd(`sls deploy`);
    });
}
exports.deploySls = deploySls;
/**
 * convert json to yml: https://www.npmjs.com/package/json-to-pretty-yaml
 *
 * @param ssrConfigPath path to a module that exports [[ServerSideRenderingConfig]]
 */
function startSlsOffline(slsConfig, keepSlsYaml) {
    return __awaiter(this, void 0, void 0, function* () {
        const fs = require("fs");
        createSlsYaml(slsConfig, keepSlsYaml);
        yield runSlsCmd(`sls offline start`, (data) => __awaiter(this, void 0, void 0, function* () {
            if (!keepSlsYaml && data.indexOf("Serverless: Offline listening") >= 0 && (yield existsSlsYml())) {
                fs.unlink('serverless.yml', function (err) {
                    if (err)
                        throw err;
                    console.log('serverless.yml removed...');
                });
            }
        }));
        /*
        await new Promise((resolve, reject) => {
            let data_line = '';
            const processRef = cmd.get(`sls offline start`);
            processRef.stdout.on(
                'data',
                async function(data) {
                    data_line += data;
                    if (data_line[data_line.length-1] == '\n') {
                        console.log(data_line);
    
                        if (!keepSlsYaml && data_line.indexOf("Serverless: Offline listening") >= 0 && await existsSlsYml()) {
                            fs.unlink('serverless.yml', function (err) {
                                if (err) throw err;
                                console.log('serverless.yml removed...');
                            });
    
                        }
    
                    }
                }
            );
    
        });*/
    });
}
exports.startSlsOffline = startSlsOffline;
exports.toSlsConfig = (stackName, serverConfig, buildPath, assetsPath, region, slsConfig) => {
    const merge = require('deepmerge');
    const path = require('path');
    return merge({
        service: {
            // it is allowed to use env-variables, but don't forget to specify them
            // e.g. "${env:CLOUDSTACKNAME}-${env:STAGE}"
            name: stackName
        },
        custom: {
            stage: "${self:provider.STAGE, env:STAGE, 'dev'}"
        },
        package: {
            include: [
                `${buildPath}/**/*`
            ],
            exclude: [
                ".infrastructure_temp/**/*",
                "build/main/**/*"
            ]
        },
        functions: {
            server: {
                // index.default refers to the default export of the file
                // this requires the server entry point to export as default:
                // `export default serverless(createServer());`
                handler: path.join(buildPath, serverConfig.name, serverConfig.name + ".default"),
                events: [
                    // this should match the public path in the app-config  
                    { http: "ANY /" },
                    { http: "'ANY {proxy+}'" },
                    { cors: "true" },
                ]
            }
        },
        // TODO put these values into the Environment-Object
        provider: {
            // set stage to environment variables
            //stage: "${env:STAGE}", done through the environment!
            // # take the region from the environment variables
            region: region,
            // we take the custom name of the CloudFormation stack from the environment variable: `CLOUDSTACKNAME`
            stackName: "${self:service.name}-${self:provider.STAGE, env:STAGE, 'dev'}",
            // Use a custom name for the API Gateway API
            apiName: "${self:service.name}-${self:provider.STAGE, env:STAGE, 'dev'}-api",
            // name of the static bucket, must match lib/getStaticBucketName
            staticBucket: `${stackName}-${assetsPath}-` + "${self:provider.STAGE, env:STAGE, 'dev'}",
            // set the environment variables
            environment: {
                // set the STAGE_PATH environment variable to the same we use during the build process
                STAGE: "${self:provider.STAGE, env:STAGE, 'dev'}",
                STAGE_PATH: "${self:provider.STAGE_PATH, env:STAGE_PATH, ''}",
                DOMAIN_URL: '{ "Fn::Join" : ["", [" https://#{ApiGatewayRestApi}", ".execute-api.' + region + '.amazonaws.com/${self:provider.STAGE, env:STAGE, \'dev\'}" ] ]  }'
            },
            // specifies the rights of the lambda-execution role
            iamRoleStatements: [
                {
                    //let the technical user read from S3
                    Effect: "Allow",
                    Action: ["s3:Get*", "s3:List*"],
                    Resource: "\"*\""
                }
            ]
        },
        resources: {
            Resources: {
                // Bucket of the client side webapp files (bundle.js...)
                StaticBucket: {
                    Type: "AWS::S3::Bucket",
                    Properties: {
                        // the bucket name is the combination of the cloudstackname, a minus, and the name of the assetsdir
                        BucketName: "${self:provider.staticBucket}",
                        AccessControl: "PublicRead",
                        WebsiteConfiguration: {
                            IndexDocument: "index.html"
                        },
                        LifecycleConfiguration: {
                            Rules: [{
                                    Id: "S3ExpireMonthly",
                                    ExpirationInDays: 30,
                                    Status: "Enabled"
                                }]
                        },
                        CorsConfiguration: {
                            CorsRules: [{
                                    AllowedMethods: ['GET'],
                                    AllowedOrigins: ['"*"'],
                                    AllowedHeaders: ['"*"']
                                }]
                        }
                    }
                },
                StaticBucketPolicy: {
                    Type: "AWS::S3::BucketPolicy",
                    Properties: {
                        Bucket: {
                            Ref: "StaticBucket"
                        },
                        PolicyDocument: {
                            Statement: {
                                Sid: "PublicReadGetObject",
                                Effect: "Allow",
                                Principal: '"*"',
                                Action: ["s3:GetObject"],
                                Resource: {
                                    "Fn::Join": "[\"\", [\"arn:aws:s3:::\", {\"Ref\": \"StaticBucket\" }, \"/*\"]]"
                                }
                            }
                        }
                    }
                },
                AssetsResource: {
                    Type: 'AWS::ApiGateway::Resource',
                    Properties: {
                        ParentId: "!GetAtt ApiGatewayRestApi.RootResourceId",
                        RestApiId: "!Ref ApiGatewayRestApi",
                        PathPart: `"${assetsPath}"`
                    }
                },
                Resource: {
                    Type: 'AWS::ApiGateway::Resource',
                    Properties: {
                        ParentId: "!Ref AssetsResource",
                        RestApiId: "!Ref ApiGatewayRestApi",
                        PathPart: '"{proxy+}"'
                    }
                },
                ProxyMethod: {
                    Type: 'AWS::ApiGateway::Method',
                    Properties: {
                        HttpMethod: "ANY",
                        ResourceId: "!Ref Resource",
                        RestApiId: "!Ref ApiGatewayRestApi",
                        AuthorizationType: "NONE",
                        RequestParameters: {
                            "method.request.path.proxy": "true"
                        },
                        Integration: {
                            CacheKeyParameters: ['"method.request.path.proxy"'],
                            RequestParameters: {
                                "integration.request.path.proxy": '"method.request.path.proxy"'
                            },
                            IntegrationHttpMethod: "ANY",
                            Type: "HTTP_PROXY",
                            Uri: "https://s3-" + region + ".amazonaws.com/${self:provider.staticBucket}/{proxy}",
                            PassthroughBehavior: "WHEN_NO_MATCH",
                            IntegrationResponses: [{
                                    StatusCode: 200
                                }]
                        }
                    }
                },
                Deployment: {
                    DependsOn: ["ProxyMethod"],
                    Type: 'AWS::ApiGateway::Deployment',
                    Properties: {
                        RestApiId: "!Ref ApiGatewayRestApi",
                        StageName: "${self:provider.STAGE, env:STAGE, 'dev'}"
                    }
                }
            }
        }
    }, slsConfig);
};
//# sourceMappingURL=sls-config.js.map