"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const webpack_libs_1 = require("./webpack-libs");
const system_libs_1 = require("./system-libs");
/**
 * The folder into which we put temporary files
 * @type {string}
 */
exports.TEMP_FOLDER = ".infrastructure_temp";
/**
 * prepares the configuration from the provided entry-file into a importable ("eval"uable) file
 *
 * @param configFilePath relative path to the webpack.config.js
 * @returns path to the created configuration file
 */
function prepareConfiguration(configFilePath) {
    return __awaiter(this, void 0, void 0, function* () {
        //console.log("loadConfiguration");
        const path = require('path');
        const absolutePath = yield system_libs_1.currentAbsolutePath();
        const webpackConfig = webpack_libs_1.complementWebpackConfig({
            entry: {
                config: "./" + configFilePath
            },
            output: {
                libraryTarget: "commonjs2",
                path: path.join(absolutePath, exports.TEMP_FOLDER),
                filename: 'config.js'
            },
            target: "node"
        });
        // pack the source code of the configuration file. This way, we include all imports/requires!
        yield webpack_libs_1.runWebpack(webpackConfig);
        // the path to the packed configuration file
        const resolvedConfigPath = path.join(absolutePath, exports.TEMP_FOLDER, 'config.js');
        return resolvedConfigPath;
    });
}
exports.prepareConfiguration = prepareConfiguration;
;
/**
 * loads the specified configuration into a Javascript object
 *
 * @param configFilePath
 */
function loadConfiguration(configFilePath) {
    const config = require(configFilePath);
    return config.default;
}
exports.loadConfiguration = loadConfiguration;
/**
 * loads the configuration as a stringified/jsonified object
 */
function loadStaticConfiguration(configFilePath) {
    // load the configuration through stringification/jsonification --> no objects but only static evaluation of definitions
    var configStr = "";
    eval(`configStr=JSON.stringify(require("${configFilePath}"), (name, val) => name ==='type' && typeof val === 'function' ? val.toString() :val)`);
    // convert the json into an object
    var config = undefined;
    eval('config=' + configStr);
    return config;
}
exports.loadStaticConfiguration = loadStaticConfiguration;
//# sourceMappingURL=configuration-lib.js.map