"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const webapp_component_1 = require("./webapp-component");
const webpack_libs_1 = require("../utils/webpack-libs");
const system_libs_1 = require("../utils/system-libs");
/**
 * A Plugin to detect WebApp-Components
 * @param props
 */
exports.WebAppPlugin = (props) => {
    const path = require('path');
    const result = {
        // identify Isomorphic-App-Components
        applies: (component) => {
            return webapp_component_1.isWebApp(component);
        },
        // convert the component into configuration parts
        // while the component is of Type `any`, its props must be of type `IWebApp`
        process: (component, childConfigs, infrastructureMode) => {
            return {
                slsConfigs: [],
                // a webapp has its own webpack configuration
                webpackConfigs: [
                    webpack_libs_1.complementWebpackConfig(webpack_libs_1.createClientWebpackConfig("./" + path.join("node_modules", "infrastructure-scripts", "assets", "client.tsx"), //entryPath: string,
                    path.join(system_libs_1.currentAbsolutePath(), props.buildPath), //use the buildpath from the parent plugin
                    component.props.id, {
                        __CONFIG_FILE_PATH__: system_libs_1.pathToConfigFile(props.configFilePath) // replace the IsoConfig-Placeholder with the real path to the main-config-bundle
                    }, {
                        WEB_APP_ID: `"${component.props.id}"` // replace the webAppId-placeholder
                    }))
                ],
                postBuilds: [],
            };
        }
    };
    return result;
};
//# sourceMappingURL=webapp-plugin.js.map